@@ -1,9 +1,10 @@
 /*
- * Copyright 2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at 
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *     http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -22,6 +23,9 @@
 package javax.jdo;
 
 import java.util.Collection;
+import java.util.Date;
+import java.util.EnumSet;
+import java.util.Set;
 
 import javax.jdo.datastore.JDOConnection;
 import javax.jdo.datastore.Sequence;
@@ -36,7 +40,7 @@ import javax.jdo.listener.InstanceLifecycleListener;
  * <P>A <code>PersistenceManager</code> is obtained from the
  * {@link PersistenceManagerFactory}
  * (recommended) or by construction.
- * @version 2.0
+ * @version 2.1
  */
 
 public interface PersistenceManager {
@@ -87,7 +91,7 @@ public interface PersistenceManager {
      * @see #evict(Object pc)
      * @param pcs the array of instances to evict from the cache.
      */
-    void evictAll (Object[] pcs);
+    void evictAll (Object... pcs);
     
     /** Mark a <code>Collection</code> of instances as no longer needed in the 
      * cache.
@@ -96,7 +100,14 @@ public interface PersistenceManager {
      * cache.
      */
     void evictAll (Collection pcs);
-    
+
+    /** Mark the parameter instances as no longer needed in the cache.
+     * @param pcClass the class of instances to evict
+     * @param subclasses if true, mark instances of subclasses also
+     * @since 2.1
+     */
+    void evictAll (boolean subclasses, Class pcClass);
+
     /** Mark all persistent-nontransactional instances as no longer needed 
      * in the cache.  It transitions
      * all persistent-nontransactional instances to hollow.  Transactional
@@ -122,7 +133,7 @@ public interface PersistenceManager {
      * @see #refresh(Object pc)
      * @param pcs the array of instances to refresh.
      */
-    void refreshAll (Object[] pcs);
+    void refreshAll (Object... pcs);
     
     /** Refresh the state of a <code>Collection</code> of instances from the 
      * data store.
@@ -253,7 +264,7 @@ public interface PersistenceManager {
      * @return an <code>Extent</code> of the specified <code>Class</code>
      * @see Query
      */
-    Extent getExtent (Class persistenceCapableClass, boolean subclasses);
+    <T> Extent<T> getExtent (Class<T> persistenceCapableClass, boolean subclasses);
 
     /**
      * Equivalent to <code>getExtent (persistenceCapableClass,
@@ -261,7 +272,7 @@ public interface PersistenceManager {
      * @see #getExtent(Class,boolean)
      * @since 2.0
      */
-    Extent getExtent (Class persistenceCapableClass);
+    <T> Extent<T> getExtent (Class<T> persistenceCapableClass);
 
     /** This method locates a persistent instance in the cache of instances
      * managed by this <code>PersistenceManager</code>.
@@ -336,7 +347,7 @@ public interface PersistenceManager {
      * @return the corresponding persistent instance
      * @since 2.0
      */
-    Object getObjectById (Class cls, Object key);
+    <T> T getObjectById (Class<T> cls, Object key);
 
     /**
      * Looks up the instance corresponding to the specified oid. This is
@@ -443,21 +454,35 @@ public interface PersistenceManager {
      * @return the objects that were looked up, in the
      *     same order as the oids parameter.
      * @see #getObjectById(Object,boolean)
+     * @see #getObjectsById(boolean,Object...)
+     * @deprecated
      * @since 2.0
      */
     Object[] getObjectsById (Object[] oids, boolean validate);
 
+    /**
+     * Return the objects with the given oids.
+     * @param oids the oids of the objects to return
+     * @param validate if true, the existance of the objects in
+     *     the datastore will be validated.
+     * @return the objects that were looked up, in the
+     *     same order as the oids parameter.
+     * @see #getObjectById(Object,boolean)
+     * @since 2.1
+     */
+    Object[] getObjectsById (boolean validate, Object... oids);
+
     /**
      * Return the objects with the given oids. This method is equivalent
-     * to calling {@link #getObjectsById(Object[],boolean)} 
+     * to calling {@link #getObjectsById(boolean,Object...)} 
      * with the validate flag true.
      * @param oids the oids of the objects to return
      * @return the objects that were looked up, in the
      *     same order as the oids parameter.
-     * @see #getObjectsById(Object[],boolean)
+     * @see #getObjectsById(boolean,Object...)
      * @since 2.0
      */
-    Object[] getObjectsById (Object[] oids);
+    Object[] getObjectsById (Object... oids);
 
     /** Make the parameter instance persistent in this 
      * <code>PersistenceManager</code>.
@@ -514,9 +539,9 @@ public interface PersistenceManager {
      * persistent state, or the corresponding persistent instance 
      * for detached parameter instances
      */
-    Object makePersistent (Object pc);
+    <T> T makePersistent (T pc);
     
-    /** Make an array of instances persistent.
+    /** Make an array of instances persistent. 
      * @param pcs an array of instances
      * @return the parameter instances for parameters in the transient or
      * persistent state, or the corresponding persistent instance 
@@ -524,7 +549,7 @@ public interface PersistenceManager {
      * parameter array
      * @see #makePersistent(Object pc)
      */
-    Object[] makePersistentAll (Object[] pcs);
+    <T> T[] makePersistentAll (T... pcs);
     
     /** Make a <code>Collection</code> of instances persistent.
      * @param pcs a <code>Collection</code> of instances
@@ -534,7 +559,7 @@ public interface PersistenceManager {
      * as in the parameter Collection
      * @see #makePersistent(Object pc)
      */
-    Collection makePersistentAll (Collection pcs);
+    <T> Collection<T> makePersistentAll (Collection<T> pcs);
     
     /** Delete the persistent instance from the data store.
      * This method must be called in an active transaction.
@@ -554,7 +579,7 @@ public interface PersistenceManager {
      * @param pcs a <code>Collection</code> of persistent instances
      * @see #deletePersistent(Object pc)
      */
-    void deletePersistentAll (Object[] pcs);
+    void deletePersistentAll (Object... pcs);
     
     /** Delete a <code>Collection</code> of instances from the data store.
      * @param pcs a <code>Collection</code> of persistent instances
@@ -580,7 +605,7 @@ public interface PersistenceManager {
      * is preserved unchanged.
      * @param pcs the instances to make transient.
      */
-    void makeTransientAll (Object[] pcs);
+    void makeTransientAll (Object... pcs);
     
     /** Make a <code>Collection</code> of instances transient, removing them 
      * from management by this <code>PersistenceManager</code>.
@@ -631,10 +656,33 @@ public interface PersistenceManager {
      * @param pcs the root instances to make transient.
      * @param useFetchPlan whether to use the current fetch plan to determine
      * which fields to load and which instances to make transient
+     * @see #makeTransientAll(boolean,Object...)
+     * @deprecated
      * @since 2.0
      */
     void makeTransientAll (Object[] pcs, boolean useFetchPlan);
     
+    /** Make instances transient, removing them from management
+     * by this <code>PersistenceManager</code>. If the useFetchPlan parameter
+     * is false, this method behaves exactly as makeTransientAll(Object[] pcs). 
+     * <P>The affected instance(s) lose their JDO identity and are no longer 
+     * associated with any <code>PersistenceManager</code>.  The state 
+     * of fields is unchanged.
+     * <P>If the useFetchPlan parameter is true, then the current FetchPlan
+     * is applied to the pcs parameters and the entire graph of instances 
+     * reachable via loaded fields is made transient. The state of fields
+     * in the affected instances is as specified by the FetchPlan.
+     * <P>Unlike detachCopy, the instances are not detached; there is no
+     * detachment information in the instances.
+     * <P>The instances to be made transient do not need to
+     * implement the javax.jdo.spi.Detachable interface.
+     * @param pcs the root instances to make transient.
+     * @param useFetchPlan whether to use the current fetch plan to determine
+     * which fields to load and which instances to make transient
+     * @since 2.1
+     */
+    void makeTransientAll (boolean useFetchPlan, Object... pcs);
+    
     /** Make instances transient, removing them from management
      * by this <code>PersistenceManager</code>. If the useFetchPlan parameter
      * is false, this method behaves exactly as 
@@ -678,7 +726,7 @@ public interface PersistenceManager {
      * @param pcs the array of instances to make transactional.
      * @see #makeTransactional(Object pc)
      */
-    void makeTransactionalAll (Object[] pcs);
+    void makeTransactionalAll (Object... pcs);
 
     /** Make a <code>Collection</code> of instances subject to transactional 
      * boundaries.
@@ -704,7 +752,7 @@ public interface PersistenceManager {
      * @param pcs the array of instances to make nontransactional.
      * @see #makeNontransactional(Object pc)
      */
-    void makeNontransactionalAll (Object[] pcs);
+    void makeNontransactionalAll (Object... pcs);
     
     /** Make a <code>Collection</code> of instances non-transactional after 
      * commit.
@@ -780,7 +828,7 @@ public interface PersistenceManager {
      * the class to retrieve associated instances.
      * @param pcs the instances
      */
-    void retrieveAll (Object[] pcs);
+    void retrieveAll (Object... pcs);
            
     /** Retrieve field values of instances from the store.  This tells
      * the <code>PersistenceManager</code> that the application intends to use 
@@ -799,10 +847,33 @@ public interface PersistenceManager {
      * @param pcs the instances
      * @param useFetchPlan whether to use the current fetch plan to determine
      * which fields to load and which instances to retrieve.
+     * @deprecated
+     * @see #retrieveAll(boolean,Object...)
      * @since 1.0.1
      */
     void retrieveAll (Object[] pcs, boolean useFetchPlan);
            
+    /** Retrieve field values of instances from the store.  This tells
+     * the <code>PersistenceManager</code> that the application intends to use 
+     * the instances, and their field values should be retrieved.  The fields
+     * in the current fetch group must be retrieved, and the implementation
+     * might retrieve more fields than the current fetch group.
+     * <P>If the useFetchPlan parameter is false, this method behaves exactly
+     * as the corresponding method without the useFetchPlan parameter. 
+     * If the useFetchPlan parameter is true, and the fetch plan has not been
+     * modified from its default setting, all fields in the current fetch plan
+     * are fetched, and other fields might be fetched lazily by the
+     * implementation. If the useFetchPlan parameter is true, and the fetch
+     * plan has been changed from its default setting, then the fields
+     * specified by the fetch plan are loaded, along with related instances
+     * specified by the fetch plan.
+     * @param pcs the instances
+     * @param useFetchPlan whether to use the current fetch plan to determine
+     * which fields to load and which instances to retrieve.
+     * @since 2.1
+     */
+    void retrieveAll (boolean useFetchPlan, Object... pcs);
+           
     /** The application can manage the <code>PersistenceManager</code> instances
      * more easily by having an application object associated with each
      * <code>PersistenceManager</code> instance.
@@ -910,6 +981,27 @@ public interface PersistenceManager {
     */
    void setDetachAllOnCommit(boolean flag);
    
+   /** Gets the copyOnAttach setting.
+    * @see #setCopyOnAttach(boolean)
+    * @since 2.1
+    * @return the copyOnAttach setting.
+    */
+   boolean getCopyOnAttach();
+
+   /** Sets the copyOnAttach setting.
+    *
+    * <P>CopyOnAttach set to <code>true</code> specifies that during
+    * makePersistent, copies are made of detached parameter instances. 
+    * With this flag set to <code>false</code>, detached parameter
+    * instances are attached directly and change their state from
+    * detached-clean to persistent-clean or from detached-dirty to
+    * persistent-dirty.
+    *
+    * @see #getCopyOnAttach()
+    * @since 2.1
+    */
+   void setCopyOnAttach(boolean flag);
+   
     /**
      * Detach the specified instance from the <code>PersistenceManager</code>.
      * The flags for detachment (DETACH_LOAD_FIELDS and DETACH_UNLOAD_FIELDS)
@@ -921,7 +1013,7 @@ public interface PersistenceManager {
      * @see #detachCopyAll(Object[])
      * @since 2.0
      */
-    Object detachCopy (Object pc);
+    <T> T detachCopy (T pc);
 
     /**
      * Detach the specified instances from the <code>PersistenceManager</code>.
@@ -934,7 +1026,7 @@ public interface PersistenceManager {
      * @see #detachCopyAll(Object[])
      * @since 2.0
      */
-    Collection detachCopyAll (Collection pcs);
+    <T> Collection<T> detachCopyAll (Collection<T> pcs);
 
     /**
      * Detach the specified instances from the <code>PersistenceManager</code>.
@@ -960,7 +1052,7 @@ public interface PersistenceManager {
      * @see #getFetchPlan
      * @since 2.0
      */
-    Object[] detachCopyAll (Object [] pcs);
+    <T> T[] detachCopyAll (T... pcs);
 
     /**
      * Put the specified key-value pair into the map of user objects.
@@ -1040,7 +1132,7 @@ public interface PersistenceManager {
      * @return the created instance
      * @since 2.0
      */
-    Object newInstance (Class pcClass);
+    <T> T newInstance (Class<T> pcClass);
 
     /**
      * Returns the sequence identified by <code>name</code>.
@@ -1075,7 +1167,7 @@ public interface PersistenceManager {
      * @since 2.0
      */
     void addInstanceLifecycleListener (InstanceLifecycleListener listener,
-        Class[] classes);
+        Class... classes);
 
     /**
      * Removes the listener instance from the list of lifecycle event listeners.
@@ -1083,4 +1175,73 @@ public interface PersistenceManager {
      * @since 2.0
      */
     void removeInstanceLifecycleListener (InstanceLifecycleListener listener);
+
+    /**
+     * Get the Date as seen by the server. 
+     * Clients using this method can order their operations according to 
+     * a single time source. Implementations use the setting of the 
+     * server time zone to prepare a Date instance that represents 
+     * UTC time on the server. 
+     * @return a Date instance corresponding to the UTC Date 
+     * as seen by the server
+     * @since 2.1
+     */
+    Date getServerDate();
+
+    /**
+     * Get the objects managed by this persistence manager.
+     * @return the objects
+     * @since 2.1
+     */
+    Set getManagedObjects();
+
+    /**
+     * Get the objects managed by this persistence manager having the
+     * specified object states.
+     * @param states The states of objects that we are interested in
+     * @return the objects
+     * @since 2.1
+     */
+    Set getManagedObjects(EnumSet<ObjectState> states);
+
+    /**
+     * Get the objects managed by this persistence manager being instances of
+     * the specified classes.
+     * @param classes The classes of objects that we are interested in
+     * @return the objects
+     * @since 2.1
+     */
+    Set getManagedObjects(Class... classes);
+
+    /**
+     * Get the objects managed by this persistence manager having the
+     * specified object states and being instances of the specified classes.
+     * @param states The states of objects that we are interested in
+     * @param classes The classes of objects that we are interested in
+     * @return the objects
+     * @since 2.1
+     */
+    Set getManagedObjects(EnumSet<ObjectState> states, Class... classes);
+
+    /**
+     * Get a modifiable <code>FetchGroup</code> for the Class and name.
+     * If a modifiable <code>FetchGroup</code> already exists in the 
+     * <code>PersistenceManager</code> scope, return it. 
+     * If not, create and populate a new <code>FetchGroup</code> from the 
+     * existing definition in the {@link PersistenceManager} or
+     * {@link PersistenceManagerFactory}. If the definition for the
+     * <code>FetchGroup</code> is not in scope in either the 
+     * <code>PersistenceManager</code> or 
+     * <code>PersistenceManagerFactory</code>, create it with no members. 
+     * The <code>FetchGroup</code> immediately 
+     * becomes active and in scope of the PersistenceManager, and hides 
+     * the corresponding fetch group in the PersistenceManagerFactory.
+     * @param cls the class or interface for the <code>FetchGroup</code>
+     * @param name the name of the fetch group
+     * @return the FetchGroup
+     * @throws JDOUserException if the class is not a persistence-capable
+     * class or interface
+     * @since 2.2
+     */
+    FetchGroup getFetchGroup(Class cls, String name);
 }
