@@ -1,9 +1,10 @@
 /*
- * Copyright 2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at 
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *     http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -87,7 +88,7 @@ import java.util.Map;
  * <code>Object</code> which must be cast to the appropriate result by the user.
  * <P>Any parameters passed to the <code>execute</code> methods are used only 
  * for this execution, and are not remembered for future execution.
- * @version 2.0
+ * @version 2.1
  */
 
 public interface Query extends Serializable {
@@ -331,7 +332,7 @@ public interface Query extends Serializable {
      * @param parameters the <code>Object</code> array with all of the 
      * parameters.
      */
-    Object executeWithArray (Object[] parameters);
+    Object executeWithArray (Object... parameters);
     
     /** Get the <code>PersistenceManager</code> associated with this 
      * <code>Query</code>.
@@ -528,7 +529,7 @@ public interface Query extends Serializable {
      * @return the number of instances of the candidate class that were deleted
      * @since 2.0
      */
-    long deletePersistentAll (Object[] parameters);
+    long deletePersistentAll (Object... parameters);
 
     /**
      * Deletes all the instances of the candidate class that pass the
@@ -583,4 +584,105 @@ public interface Query extends Serializable {
       */
     boolean isUnmodifiable();
 
+    /**
+     * Add a subquery to this query.
+     * @param sub the subquery to add to this Query
+     * @param variableDeclaration the name of the variable in the outer query
+     * to bind the results of the subquery
+     * @param candidateCollectionExpression the candidate collection 
+     * of the subquery as an expression using terms of the outer query
+     * @see #addSubquery(Query sub, String variableDeclaration, 
+     *    String candidateCollectionExpression, String... parameters)
+     * @since 2.1
+     */
+    void addSubquery
+        (Query sub, String variableDeclaration, 
+            String candidateCollectionExpression);
+
+    /**
+     * Add a subquery to this query.
+     * The String version of the method binds the named expression 
+     * to the parameter implictly or explicitly declared in the 
+     * subquery.
+     * @param sub the subquery to add to this Query
+     * @param variableDeclaration the name of the variable 
+     * to be used in this Query
+     * @param candidateCollectionExpression the candidate collection 
+     * to apply to the subquery
+     * @param parameter the expression from the outer query to bind 
+     * the parameter in the subquery
+     * @see #addSubquery(Query sub, String variableDeclaration, 
+     *    String candidateCollectionExpression, String... parameters)
+     * @since 2.1
+     */
+    void addSubquery
+        (Query sub, String variableDeclaration, 
+         String candidateCollectionExpression, String parameter);
+
+    /**
+     * Add a subquery to this query.
+     * A subquery is composed as a Query and subsequently attached
+     * to a different query (the outer query) by calling this method.
+     * The query parameter instance is unmodified as a result of the
+     * addSubquery or subsequent execution of the outer query.
+     * Only some of the query parts are copied for use as the subquery.
+     * The parts copied include the candidate class, filter, parameter
+     * declarations, variable declarations, imports, ordering specification,
+     * uniqueness, result specification, and grouping specification.
+     * The association with a PersistenceManager, the candidate collection
+     * or extent, result class, and range limits are not used.
+     * The String parameters are trimmed of white space.
+     * The variableDeclaration parameter is the name of the variable
+     * containing the results of the subquery execution. If the same value
+     * of variableDeclaration is used to add multiple subqueries, the
+     * subquery replaces the previous subquery for the same named variable.
+     * If the subquery parameter is null, the variable is unset,
+     * effectively making the variable named in the variableDeclaration
+     * unbound. If the trimmed value is the empty String, or the parameter
+     * is null, then JDOUserException is thrown.
+     * The candidateCollectionExpression is the expression from the
+     * outer query that represents the candidates over which the subquery
+     * is evaluated. If the trimmed value is the empty String, or the
+     * parameter is null, then the candidate collection is the extent
+     * of the candidate class.
+     * The String... version of the method binds the named expressions in 
+     * turn to parameters in the order in which they are declared in the 
+     * subquery, or in the order they are found in the filter if not 
+     * explicitly declared in the subquery.
+     * @param sub the subquery to add to this Query
+     * @param variableDeclaration the name of the variable in the outer query
+     * to bind the results of the subquery
+     * @param candidateCollectionExpression the candidate collection 
+     * of the subquery as an expression using terms of the outer query
+     * @param parameters the expressions from the outer query to bind 
+     * the parameters in the subquery
+     * @since 2.1
+     */
+    void addSubquery
+        (Query sub, String variableDeclaration, 
+         String candidateCollectionExpression, String... parameters);
+
+    /**
+     * Add a subquery to this query.
+     * The Map version of the method treats the key of each map entry as 
+     * the name of  the parameter in the subquery, with or without the 
+     * leading ":", and the value as the name of the expression in the 
+     * outer query. If the trimmed expression is the empty String for 
+     * either the parameter or the value of the String[], or for any 
+     * map key or value, that expression is ignored.
+     * @param sub the subquery to add to this Query
+     * @param variableDeclaration the name of the variable 
+     * to be used in this Query
+     * @param candidateCollectionExpression the candidate collection 
+     * to apply to the subquery
+     * @param parameters the expressions from the outer query to bind 
+     * the parameter in the subquery
+     * @see #addSubquery(Query sub, String variableDeclaration, 
+     *    String candidateCollectionExpression, String... parameters)
+     * @since 2.1
+     */
+    void addSubquery
+        (Query sub, String variableDeclaration, 
+         String candidateCollectionExpression, Map parameters);
+
 }
