@@ -1,9 +1,10 @@
 /*
- * Copyright 2005 The Apache Software Foundation.
- * 
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at 
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  * 
  *     http://www.apache.org/licenses/LICENSE-2.0
  * 
@@ -21,8 +22,13 @@
 
 package javax.jdo.spi;
 
+import org.xml.sax.ErrorHandler;
+
 import java.lang.reflect.Constructor;
 
+import java.security.AccessController;
+import java.security.PrivilegedAction;
+
 import java.text.DateFormat;
 import java.text.ParsePosition;
 import java.text.SimpleDateFormat;
@@ -44,6 +50,7 @@ import javax.jdo.JDOException;
 import javax.jdo.JDOFatalInternalException;
 import javax.jdo.JDOFatalUserException;
 import javax.jdo.JDOUserException;
+import javax.xml.parsers.DocumentBuilderFactory;
 
 /** This class is a helper class for JDO implementations.  It contains methods
  * to register metadata for persistence-capable classes and to perform common
@@ -58,7 +65,7 @@ import javax.jdo.JDOUserException;
  * the security manager, only the call to get an instance is checked.  Once an 
  * implementation
  * has an instance, any of the methods can be invoked without security checks.
- * @version 1.0.2
+ * @version 2.1
  *
  */
 public class JDOImplHelper extends java.lang.Object {
@@ -103,10 +110,20 @@ public class JDOImplHelper extends java.lang.Object {
      */
     private static DateFormat dateFormat;
 
+    /**
+     * The DocumentBuilderFactory used during jdoconfig.xml parsing.
+     */
+    private static DocumentBuilderFactory documentBuilderFactory;
+
+    /**
+     * The ErrorHandler used during jdoconfig.xml parsing.
+     */
+    private static ErrorHandler errorHandler;
+
     /** Register the default DateFormat instance.
      */
     static {
-        jdoImplHelper.registerDateFormat(DateFormat.getDateTimeInstance());
+        jdoImplHelper.registerDateFormat(getDateTimeInstance());
     }
     
     /** Creates new JDOImplHelper */
@@ -529,6 +546,49 @@ public class JDOImplHelper extends java.lang.Object {
         }
     }
     
+    /**
+     * Register a DocumentBuilderFactory instance for use in parsing the
+     * resource(s) META-INF/jdoconfig.xml. The default is governed by the
+     * semantics of DocumentBuilderFactory.newInstance().
+     *
+     * @param factory the DocumentBuilderFactory instance to use
+     * @since 2.1
+     */
+    public synchronized void registerDocumentBuilderFactory(
+            DocumentBuilderFactory factory) {
+        documentBuilderFactory = factory;
+    }
+
+    /**
+     * Return the registered instance of DocumentBuilderFactory.
+     * @return the DocumentBuilderFactory if registered; null otherwise
+     * @since 2.1
+     */
+    public static DocumentBuilderFactory getRegisteredDocumentBuilderFactory() {
+        return documentBuilderFactory;
+    }
+
+    /**
+     * Register an ErrorHandler instance for use in parsing the
+     * resource(s) META-INF/jdoconfig.xml. The default is an ErrorHandler
+     * that throws on error or fatalError and ignores warnings.
+     *
+     * @param handler the ErrorHandler instance to use
+     * @since 2.1
+     */
+    public synchronized void registerErrorHandler(ErrorHandler handler) {
+        errorHandler = handler;
+    }
+
+    /**
+     * Return the registered instance of ErrorHandler.
+     * @return the registered ErrorHandler if registered; null otherwise
+     * @since 2.1
+     */
+    public static ErrorHandler getRegisteredErrorHandler() {
+        return errorHandler;
+    }
+
     /** Check that the parameter instance is of a class that is authorized for
      * JDOPermission("setStateManager").  This method is called by the
      * replaceStateManager method in persistence-capable classes.
@@ -609,9 +669,8 @@ public class JDOImplHelper extends java.lang.Object {
     /** Register the default special StringConstructor instances.
      */
     static {
-        JDOImplHelper helper = getInstance();
         if (isClassLoadable("java.util.Currency")) {
-            helper.registerStringConstructor(
+            jdoImplHelper.registerStringConstructor(
                     Currency.class, new StringConstructor() {
                 public Object construct(String s) {
                     try {
@@ -628,7 +687,7 @@ public class JDOImplHelper extends java.lang.Object {
                 }
             });
         }
-        helper.registerStringConstructor(Locale.class, new StringConstructor() {
+        jdoImplHelper.registerStringConstructor(Locale.class, new StringConstructor() {
             public Object construct(String s) {
                 try {
                     return getLocale(s);
@@ -638,7 +697,7 @@ public class JDOImplHelper extends java.lang.Object {
                 }
             }
         });
-        helper.registerStringConstructor(Date.class, new StringConstructor() {
+        jdoImplHelper.registerStringConstructor(Date.class, new StringConstructor() {
             public synchronized Object construct(String s) {
                 try {
                     // first, try the String as a Long
@@ -661,6 +720,8 @@ public class JDOImplHelper extends java.lang.Object {
     
     /**
      * Parse the String to a Locale.
+     * @param s the name of the Locale
+     * @return the Locale corresponding to the name
      */
     private static Locale getLocale(String s) {
         String lang = s;
@@ -683,6 +744,8 @@ public class JDOImplHelper extends java.lang.Object {
     }
     /**
      * Determine if a class is loadable in the current environment.
+     * @param className the fully-qualified name of the class
+     * @return true if the class can be loaded; false otherwise
      */
     private static boolean isClassLoadable(String className) {
         try {
@@ -732,11 +795,40 @@ public class JDOImplHelper extends java.lang.Object {
         }
     }
 
+    /**
+     * Get the DateFormat instance for the default locale from the VM.
+     * This requires the following privileges for JDOImplHelper in the
+     * security permissions file:
+     * permission java.util.PropertyPermission "user.country", "read";
+     * permission java.util.PropertyPermission "user.timezone", "read,write";
+     * permission java.util.PropertyPermission "java.home", "read";
+     * If these permissions are not present, or there is some other
+     * problem getting the default date format, a simple formatter is returned.
+     * @since 2.1
+     * @return the default date-time formatter
+     */
+    static DateFormat getDateTimeInstance() {
+        DateFormat result = null;
+        try {
+        result = (DateFormat) AccessController.doPrivileged (
+            new PrivilegedAction () {
+                public Object run () {
+                    return DateFormat.getDateTimeInstance();
+                }
+            }
+            );
+        } catch (Exception ex) {
+            result = DateFormat.getInstance();
+        }
+        return result;
+    }
+
     /**
      * Register a DateFormat instance for use with constructing Date 
      * instances. The default is the default DateFormat instance.
      * If the new instance implements SimpleDateFormat, get its pattern
      * for error messages.
+     * @since 2.0
      * @param df the DateFormat instance to use
      */
     public synchronized void registerDateFormat(DateFormat df) {
@@ -845,8 +937,10 @@ public class JDOImplHelper extends java.lang.Object {
         }
     }
     
-    /** Add a StateInterrogation to the list. Create a new list
+    /**
+     * Add a StateInterrogation to the list. Create a new list
      * in case there is an iterator open on the original list.
+     * @param si the StateInterrogation to add
      */
     public synchronized void addStateInterrogation(StateInterrogation si) {
         List newList = new ArrayList(stateInterrogations);
@@ -854,8 +948,10 @@ public class JDOImplHelper extends java.lang.Object {
         stateInterrogations = newList;
     }
     
-    /** Remove a StateInterrogation from the list. Create a new list
+    /**
+     * Remove a StateInterrogation from the list. Create a new list
      * in case there is an iterator open on the original list.
+     * @param si the StateInterrogation to remove
      */
     public synchronized void removeStateInterrogation(StateInterrogation si) {
         List newList = new ArrayList(stateInterrogations);
@@ -863,55 +959,87 @@ public class JDOImplHelper extends java.lang.Object {
         stateInterrogations = newList;
     }
     
-    /** Return an Iterator over all StateInterrogation instances.
+    /**
+     * Return an Iterator over all StateInterrogation instances.
      * Synchronize to avoid add/remove/iterate conflicts.
+     * @return an Iterator over all StateInterrogation instances.
      */
     private synchronized Iterator getStateInterrogationIterator() {
         return stateInterrogations.iterator();
     }
     
-    /** Mark a non-binary-compatible instance dirty. Delegate to all
+    /**
+     * Mark a non-binary-compatible instance dirty. Delegate to all
      * registered StateInterrogation instances until one of them
      * handles the call.
+     * @param pc the instance to mark dirty
+     * @param fieldName the field to mark dirty
      */
     public void nonBinaryCompatibleMakeDirty(Object pc, String fieldName) {
         Iterator sit = getStateInterrogationIterator();
         while (sit.hasNext()) {
             StateInterrogation si = (StateInterrogation)sit.next();
-            if (si.makeDirty(pc, fieldName)) return;
+            try {
+                if (si.makeDirty(pc, fieldName)) return;
+            } catch (Throwable t) {
+                continue; // ignore exceptions from errant StateInterrogations
+            }
         }
     }
     
-    /** Determine the state of a non-binary-compatible instance.
+    /**
+     * Determine the state of a non-binary-compatible instance.
      * Delegate to all registered StateInterrogation instances until
      * one of them handles the call (returns a non-null Boolean 
      * with the answer).
      * The caller provides the stateless "method object" that does 
      * the actual call to the StateInterrogation instance.
+     * @param pc the instance to be checked
+     * @param sibr the method object that delegates to the 
+     * non-binary-compatible implementation
+     * @return Boolean.TRUE if the instance satisfies the state interrogation;
+     * Boolean.FALSE if the instance does not satisfy the interrogation;
+     * or null if the implementation does not manage the class of the instance
      */
     public boolean nonBinaryCompatibleIs(Object pc, 
             StateInterrogationBooleanReturn sibr) {
         Iterator sit = getStateInterrogationIterator();
         while (sit.hasNext()) {
             StateInterrogation si = (StateInterrogation)sit.next();
-            Boolean result = sibr.is(pc, si);
+            Boolean result;
+            try {
+                result = sibr.is(pc, si);
+            } catch (Throwable t) {
+                continue; // ignore exceptions from errant StateInterrogations
+            }
             if (result != null) return result.booleanValue();
         }
         return false;
     }
     
-    /** Return an object associated with a non-binary-compatible instance.
+    /**
+     * Return an object associated with a non-binary-compatible instance.
      * Delegate to all registered StateInterrogation instances until
      * one of them handles the call (returns a non-null answer).
      * The caller provides the stateless "method object" that does 
      * the actual call to the StateInterrogation instance.
+     * @param pc the instance whose associated object is needed
+     * @param sibr the method object that delegates to the 
+     * non-binary-compatible implementation
+     * @return the associated object or null if the implementation does not
+     * manage the class of the instance
      */
     public Object nonBinaryCompatibleGet(Object pc, 
             StateInterrogationObjectReturn sibr) {
         Iterator sit = getStateInterrogationIterator();
         while (sit.hasNext()) {
             StateInterrogation si = (StateInterrogation)sit.next();
-            Object result = sibr.get(pc, si);
+            Object result;
+            try {
+                result = sibr.get(pc, si);
+            } catch (Throwable t) {
+                continue; // ignore exceptions from errant StateInterrogations
+            }
             if (result != null) return result;
         }
         return null;
@@ -922,6 +1050,12 @@ public class JDOImplHelper extends java.lang.Object {
      * methods that return a boolean value.
      */
     public static interface StateInterrogationBooleanReturn {
+        /**
+         * Interrogate the state of the instance 
+         * @param pc the instance
+         * @param si the method object
+         * @return the state of the instance or null
+         */
         public Boolean is(Object pc, StateInterrogation si);
     }
     
@@ -930,6 +1064,12 @@ public class JDOImplHelper extends java.lang.Object {
      * methods that return an Object value.
      */
     public static interface StateInterrogationObjectReturn {
+        /**
+         * Return the associated instance.
+         * @param pc the instance
+         * @param si the method object
+         * @return the associated object or null
+         */
         public Object get(Object pc, StateInterrogation si);
     }
 }
